function AutoLink_Load()
	al_version = "0.4.1"
	DEFAULT_CHAT_FRAME:AddMessage("AutoLink v"..al_version.." by ogreinside loaded.", 0, 1, 0)
	if not al then
		AutoLink_Reset()
	else	-- These were added in v0.2 and beyond
		if al.trade == nil then al.trade = false end
		if al.general == nil then al.general = false end
		if al.alchemy == nil then al.alchemy = true end
		if al.bs == nil then al.bs = true end
		if al.cooking == nil then al.cooking = true end
		if al.enchanting == nil then al.enchanting = true end
		if al.eng == nil then al.eng = true end
		if al.firstaid == nil then al.firstaid = false end
		if al.inscription == nil then al.inscription = true end
		if al.jc == nil then al.jc = true end
		if al.lw == nil then al.lw = true end
		if al.tailoring == nil then al.tailoring = true end
		if al.self == nil then al.self = false end
		if al.debug == nil then al.debug = false end
		if al.append == nil then al.append = "" end
		if al.always == nil then al.always = false end
		if al.ignore == nil then al.ignore = {} end
		if al.debug == nil then AutoLink_Debug() end
		-- Added 0.4
		if al.prefix == nil then al.prefix = "(AutoLink) " end
		if al.nospam == nil then al.nospam = true end
		if al.public == nil then al.public = false end
		if al.all == nil then al.all = true end
	end
	linked = {}
end

function AutoLink_DebugChat(chat_msg)
	if al.debug then 
		DEFAULT_CHAT_FRAME:AddMessage("AutoLink DEBUG: "..chat_msg, 0, 1, 0);
	end
end

function AutoLink_RespondChat(chat_msg, chat_type, chat_requestor)
	AutoLink_DebugChat("DEBUG: chat type: "..chat_type.." requestor: "..chat_requestor)
	if al.public then	--If al.public is true, respond to original channel
		if (string.find(chat_type, "Trade") or string.find(chat_type, "General")) then
			local index, name = GetChannelName(chat_type)
			if not (index==nil or index==0) then 
				AutoLink_DebugChat("DEBUG: sending chat to CHANNEL: "..index)
				SendChatMessage(chat_msg, "CHANNEL", nil, index)
			elseif index==0 then 	--Probably General failing, as happend to me often
				AutoLink_DebugChat("DEBUG: sending chat "..chat_type.." to CHANNEL: "..index)
				SendChatMessage(chat_msg, "WHISPER", nil, chat_requestor)
			end
		else
			SendChatMessage(chat_msg, chat_type, nil, chat_requestor)
		end
	else	--Since al.public is false, only generate a tell
		SendChatMessage(chat_msg, "WHISPER", nil, chat_requestor)
		return
	end
end

function AutoLink_GetSkill(skill)
	return select(2, GetSpellLink(skill))
end

function AutoLink_NoSkill(skill, return_to_sender)
	AutoLink_RespondChat("Sorry, I do not have "..skill, "WHISPER", return_to_sender)
	return
end

function AutoLink_Reset()
	al = {
		enabled = true,
		party = true,
		raid = true,
		guild = true,
		say = false,
		yell = false,
		trade = false,
		general = false,
		alchemy = true,
		bs = true,
		cooking = true,
		enchanting = true,
		eng = true,
		firstaid = false,  -- No real practical reason to link First Aid
		inscription = true,
		jc = true,
		lw = true,
		tailoring = true,
		self = false,
		always = false,
		debug = false,
		ignore = {},
		message = "",
		prefix = "(AutoLink) ",
		nospam = true,
		all = true
	}
end

function AutoLink_Debug()
	AutoLink_DebugChat("value of party: " .. tostring(al.party))
	AutoLink_DebugChat("value of raid: " .. tostring(al.raid))
	AutoLink_DebugChat("value of guild: " .. tostring(al.guild))
	AutoLink_DebugChat("value of say: " .. tostring(al.say))
	AutoLink_DebugChat("value of yell: " .. tostring(al.yell))
	AutoLink_DebugChat("value of trade: " .. tostring(al.trade))
	AutoLink_DebugChat("value of general: " .. tostring(al.general))
	AutoLink_DebugChat("value of alchemy: " .. tostring(al.alchemy))
	AutoLink_DebugChat("value of bs: " .. tostring(al.bs))
	AutoLink_DebugChat("value of cooking: " .. tostring(al.cooking))
	AutoLink_DebugChat("value of enchanting: " .. tostring(al.enchanting))
	AutoLink_DebugChat("value of eng: " .. tostring(al.eng))
	AutoLink_DebugChat("value of firstaid: " .. tostring(al.firstaid))
	AutoLink_DebugChat("value of inscription: " .. tostring(al.inscription))
	AutoLink_DebugChat("value of jc: " .. tostring(al.jc))
	AutoLink_DebugChat("value of lw: " .. tostring(al.lw))
	AutoLink_DebugChat("value of tailoring: " .. tostring(al.tailoring))
	AutoLink_DebugChat("value of self: " .. tostring(al.self))
	AutoLink_DebugChat("value of nospam: " .. tostring(al.nospam))
	AutoLink_DebugChat("value of public: " .. tostring(al.public))
	AutoLink_DebugChat("value of all: " .. tostring(al.all))
	AutoLink_DebugChat("link prefix: " .. tostring( al.prefix))
	AutoLink_DebugChat("custom message: " .. tostring( al.append))
	for name,status in pairs(al.ignore) do
		AutoLink_DebugChat("Ignore list entry: " .. tostring( name))
	end
end

function AutoLink_List()
	if al.debug then DEFAULT_CHAT_FRAME:AddMessage("AutoLink DEBUG: Compiling monitored/ignored channel lists.", 0, 1, 0) end
	DEFAULT_CHAT_FRAME:AddMessage("AutoLink v"..al_version.." by ogreinside.", 0, 1, 0)
	DEFAULT_CHAT_FRAME:AddMessage("AutoLink channel monitoring status:", 0, 1, 0)
	channels_monitored = ""
	channels_ignored = ""
	if al.guild then
		channels_monitored = channels_monitored .. "|cFFFFFFFFguild|r "
	else
		channels_ignored = channels_ignored .. "|cFFFFFFFFguild|r "
	end
	if al.party then
		channels_monitored = channels_monitored .. "|cFFFFFFFFparty|r "
	else
		channels_ignored = channels_ignored .. "|cFFFFFFFFparty|r "
	end
	if al.raid then
		channels_monitored = channels_monitored .. "|cFFFFFFFFraid|r "
	else
		channels_ignored = channels_ignored .. "|cFFFFFFFFraid|r "
	end
	if al.say then
		channels_monitored = channels_monitored .. "|cFFFFFFFFsay|r "
	else
		channels_ignored = channels_ignored .. "|cFFFFFFFFsay|r "
	end
	if al.yell then
		channels_monitored = channels_monitored .. "|cFFFFFFFFyell|r "
	else
		channels_ignored = channels_ignored .. "|cFFFFFFFFyell|r "
	end
	if al.trade then
		channels_monitored = channels_monitored .. "|cFFFFFFFFtrade|r "
	else
		channels_ignored = channels_ignored .. "|cFFFFFFFFtrade|r "
	end
	if al.general then
		channels_monitored = channels_monitored .. "|cFFFFFFFFgeneral|r "
	else
		channels_ignored = channels_ignored .. "|cFFFFFFFFgeneral|r "
	end
	if al.self then
		channels_monitored = channels_monitored .. "|cFFFFFFFFself|r "
	else
		channels_ignored = channels_ignored .. "|cFFFFFFFFself|r "
	end
	if not al.always then
		channels_monitored = channels_monitored .. "|cFFFFFFFFafk/dnd_status|r "
	else
		channels_ignored = channels_ignored .. "|cFFFFFFFFafk/dns_status|r "
	end
	if al.all then
		channels_monitored = channels_monitored .. "|cFFFFFFFFall_skills|r "
	else
		channels_ignored = channels_ignored .. "|cFFFFFFFFall_skills|r "
	end
	if channels_monitored == "" then
		channels_monitored = "|cFFFFFFFFNone|r"
	end
	if channels_ignored == "" then
		channels_ignored = "|cFFFFFFFFNone|r"
	end
	DEFAULT_CHAT_FRAME:AddMessage("  |cFF0000FFMonitoring|r: " .. channels_monitored, 0, 1, 0)
	DEFAULT_CHAT_FRAME:AddMessage("  |cFFFF0000Ignoring|r: " .. channels_ignored, 0, 1, 0)
	if al.prefix ~= "" then
		DEFAULT_CHAT_FRAME:AddMessage("  |cFFFFFF00Prefix|r: " .."|cFFFFFFFF".. al.prefix .. "|r", 0, 1, 0)
	end
	if al.append ~= "" then
		DEFAULT_CHAT_FRAME:AddMessage("  |cFFFFFF00Message|r: " .."|cFFFFFFFF".. al.append.."|r", 0, 1, 0)
	end
	if al.nospam then
		DEFAULT_CHAT_FRAME:AddMessage("  |cFFFFFF00Anti-Spam|r: " .."|cFFFFFFFFOn|r", 0, 1, 0)
	else
		DEFAULT_CHAT_FRAME:AddMessage("  |cFFFFFF00Anti-Spam|r: " .."|cFFFFFFFFOff|r", 0, 1, 0)
	end
	if al.public then
		DEFAULT_CHAT_FRAME:AddMessage("  |cFFFFFF00Respond|r: " .."|cFFFFFFFFSend to original channel|r", 0, 1, 0)
	else
		DEFAULT_CHAT_FRAME:AddMessage("  |cFFFFFF00Respond|r: " .."|cFFFFFFFFSend tell only|r", 0, 1, 0)
	end
end

function AutoLink_GetAllCrafts()
	AutoLink_DebugChat("Compiling list of all known crafts.")
	local skills = {}
	if al.alchemy then
		skill = AutoLink_GetSkill("Alchemy")
		table.insert(skills, skill)
	end
	if al.bs then
		skill = AutoLink_GetSkill("Blacksmithing")
		table.insert(skills, skill)
	end
	if al.enchanting then
		skill = AutoLink_GetSkill("Enchanting")
		table.insert(skills, skill)
	end
	if al.eng then
		skill = AutoLink_GetSkill("Engineering")
		table.insert(skills, skill)
	end
	if al.inscription then
		skill = AutoLink_GetSkill("Inscription")
		table.insert(skills, skill)
	end
	if al.jc then
		skill = AutoLink_GetSkill("Jewelcrafting")
		table.insert(skills, skill)
	end
	if al.lw then
		skill = AutoLink_GetSkill("Leatherworking")
		table.insert(skills, skill)
	end
	if al.tailoring then
		skill = AutoLink_GetSkill("Tailoring")
		table.insert(skills, skill)
	end
	if al.cooking then
		skill = AutoLink_GetSkill("Cooking")
		table.insert(skills, skill)
	end
	if al.firstaid then 
		skill = AutoLink_GetSkill("First Aid")
		table.insert(skills, skill)
	end
	return skills
end

function AutoLink_GetCmd(msg)
 	if msg then
 		local a,b,c=strfind(msg, "(%S+)"); 
 		if a then
 			return c, strsub(msg, b+2)
 		else	
 			return ""
 		end
 	end
end

function AutoLink_GetArgument(msg)
 	if msg then
 		local a,b=strfind(msg, "=")
 		if a then
 			return strsub(msg,1,a-1), strsub(msg, b+1)
 		else	
 			return ""
 		end
 	end
end

function AutoLink_SlashCmdHandler(slashcmd)
	local maincmd, subcmd = AutoLink_GetCmd(slashcmd)
	cmd = strlower(maincmd)
	if al.debug then DEFAULT_CHAT_FRAME:AddMessage("AutoLink DEBUG: Processing commands: ["..cmd.. "], ["..subcmd.."]", 0, 1, 0) end
	if InCombatLockdown() then
		DEFAULT_CHAT_FRAME:AddMessage("AutoLink - Not operable during combat.", 0, 1, 0)
		return
	elseif (cmd == "enable" or cmd == "on") then
		if al.enabled then
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink is already enabled.", 0, 1, 0)
		else
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink is now enabled.", 0, 1, 0)
			al.enabled = true
		end
	elseif not al.enabled then
		DEFAULT_CHAT_FRAME:AddMessage("AutoLink is disabled, no other commands are possible.", 0, 1, 0)
		return
	elseif (cmd == "disable" or cmd == "off") then
		if not al.enabled then
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink is already disabled.", 0, 1, 0)
		else
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink is now disabled.", 0, 1, 0)
			al.enabled = false
		end
	elseif cmd == "party" then
		if al.party then
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now ignore party chat.", 0, 1, 0)
			al.party = false
		else
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now monitor party chat.", 0, 1, 0)
			al.party = true
		end
	elseif cmd == "guild" then
		if al.guild then
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now ignore guild chat.", 0, 1, 0)
			al.guild = false
		else
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now monitor guild chat.", 0, 1, 0)
			al.guild = true
		end
	elseif cmd == "raid" then
		if al.raid then
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now ignore raid chat.", 0, 1, 0)
			al.raid = false
		else
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now monitor raid chat.", 0, 1, 0)
			al.raid = true
		end
	elseif cmd == "say" then
		if al.say then
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now ignore say chat.", 0, 1, 0)
			al.say = false
		else
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now monitor say chat.", 0, 1, 0)
			al.say = true
		end
	elseif cmd == "yell" then
		if al.yell then
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now ignore yell chat.", 0, 1, 0)
			al.yell = false
		else
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now monitor yell chat.", 0, 1, 0)
			al.yell = true
		end
	elseif cmd == "general" then
		if al.general then
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now ignore general chat.", 0, 1, 0)
			al.general = false
		else
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now monitor general chat.", 0, 1, 0)
			al.general = true
		end
	elseif cmd == "trade" then
		if al.trade then
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now ignore trade chat.", 0, 1, 0)
			al.trade = false
		else
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now monitor trade chat.", 0, 1, 0)
			al.trade = true
		end
	elseif cmd == "alchemy" then
		if not IsUsableSpell("Alchemy") then
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink - You do not have the Alchemy skill.", 0, 1, 0)
			return
		end
		if al.alchemy then
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now ignore Alchemy skill.", 0, 1, 0)
			al.alchemy = false
		else
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now link Alchemy skill.", 0, 1, 0)
			al.alchemy = true
		end
	elseif (cmd == "bs" or cmd == "blacksmithing") then
		if not IsUsableSpell("Blacksmithing") then
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink - You do not have the Blacksmithing skill.", 0, 1, 0)
			return
		end
		if al.bs then
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now ignore Blacksmithing skill.", 0, 1, 0)
			al.bs = false
		else
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now link Blacksmithing skill.", 0, 1, 0)
			al.bs = true
		end
	elseif (cmd == "cooking" or cmd == "cook") then
		if not IsUsableSpell("Cooking") then
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink - You do not have the Cooking skill.", 0, 1, 0)
			return
		end
		if al.cooking then
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now ignore Cooking skill.", 0, 1, 0)
			al.cooking = false
		else
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now link Cooking skill.", 0, 1, 0)
			al.cooking = true
		end
	elseif (cmd == "enchanting" or cmd == "enchant") then
		if not IsUsableSpell("Enchanting") then
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink - You do not have the Enchanting skill.", 0, 1, 0)
			return
		end
		if al.enchanting then
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now ignore Enchanting skill.", 0, 1, 0)
			al.enchanting = false
		else
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now link Enchanting skill.", 0, 1, 0)
			al.enchanting = true
		end
	elseif (cmd == "eng" or cmd == "engineering") then
		if not IsUsableSpell("Engineering") then
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink - You do not have the Engineering skill.", 0, 1, 0)
			return
		end
		if al.eng then
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now ignore Engineering skill.", 0, 1, 0)
			al.eng = false
		else
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now link Engineering skill.", 0, 1, 0)
			al.eng = true
		end
	elseif cmd == "firstaid" then
		if not IsUsableSpell("First Aid") then
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink - You do not have the First Aid skill.", 0, 1, 0)
			return
		end
		if al.firstaid then
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now ignore First Aid skill.", 0, 1, 0)
			al.firstaid = false
		else
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now link First Aid skill.", 0, 1, 0)
			al.firstaid = true
		end
	elseif cmd == "inscription" then
		if not IsUsableSpell("Inscription") then
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink - You do not have the Inscription skill.", 0, 1, 0)
			return
		end
		if al.inscription then
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now ignore Inscription skill.", 0, 1, 0)
			al.inscription = false
		else
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now link Inscription skill.", 0, 1, 0)
			al.inscription = true
		end
	elseif (cmd == "jc" or cmd == "jewelcrafting" or cmd == "jewelcraft") then
		if not IsUsableSpell("Jewelcrafting") then
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink - You do not have the Jewelcrafting skill.", 0, 1, 0)
			return
		end
		if al.jc then
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now ignore Jewelcrafting skill.", 0, 1, 0)
			al.jc = false
		else
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now link Jewelcrafting skill.", 0, 1, 0)
			al.jc = true
		end
	elseif (cmd == "lw" or cmd == "leatherworking" or cmd == "leather") then
		if not IsUsableSpell("Leatherworking") then
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink - You do not have the Leatherworking skill.", 0, 1, 0)
			return
		end
		if al.lw then
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now ignore Leatherworking skill.", 0, 1, 0)
			al.lw = false
		else
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now link Leatherworking skill.", 0, 1, 0)
			al.lw = true
		end
	elseif (cmd == "tailoring" or cmd == "tailor") then
		if not IsUsableSpell("Tailoring") then
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink - You do not have the Tailoring skill.", 0, 1, 0)
			return
		end
		if al.tailoring then
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now ignore Tailoring skill.", 0, 1, 0)
			al.tailoring = false
		else
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now link Tailoring skill.", 0, 1, 0)
			al.tailoring = true
		end
	elseif cmd == "all_chats" then
			al.party = true
			al.raid = true
			al.guild = true
			al.say = true
			al.yell = true
			al.general = true
			al.trade = true
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now monitor all chat channels.", 0, 1, 0)
	elseif (cmd == "self" or cmd == "me") then
		if al.self then
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now ignore your own chat.", 0, 1, 0)
			al.self = false
		else
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now respond to your own chat.", 0, 1, 0)
			al.self = true
		end
	elseif cmd == "nospam" then
		if al.nospam then
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink now doesn't care about spamming, use at your own risk :)", 0, 1, 0)
			al.nospam = false
		else
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now ban people for 1 minute to prevent spamming, and only send tells for Trade/General.", 0, 1, 0)
			al.nospam = true
		end
	elseif cmd == "public" then
		if al.public then
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now only generate a tell to the requestor.", 0, 1, 0)
			al.public = false
		else
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now respond to the original channel the request came from.", 0, 1, 0)
			al.public = true
		end
	elseif cmd == "all" then
			DEFAULT_CHAT_FRAME:AddMessage("As of version 0.5, you must specify 'all_chats' or 'all_skills'.", 0, 1, 0)
	elseif cmd == "all_skills" then
		if al.all then
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will ignore requests for all skills.", 0, 1, 0)
			al.all = false
		else
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now respond to requests for all skills.", 0, 1, 0)
			al.all = true
		end
	elseif cmd == "always" then
		if al.always then
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now only work if you are not AFK/DND.", 0, 1, 0)
			al.always = false
		else
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now continue to work even if you are AFK/DND.", 0, 1, 0)
			al.always = true
		end
	elseif cmd == "list" then
		AutoLink_List()
		local skills = AutoLink_GetAllCrafts()
		if (skills==nil or skills=="") then 		--no professions, or all toggled off
			all_skills = "|cFFFFFFFFNone|r"
		else
			all_skills = ""
			for i,skill in pairs(skills) do
				all_skills = all_skills .. skill
			end
		end
		DEFAULT_CHAT_FRAME:AddMessage("  |cFFFFFFFFLinking|r: " .. all_skills, 0, 1, 0)
	elseif (cmd == "reset" or cmd == "default") then
		AutoLink_Reset()
		DEFAULT_CHAT_FRAME:AddMessage("AutoLink settings have been reset to defaults.", 0, 1, 0)
	elseif cmd == "prefix" then
		if (subcmd == "" or subcmd == nil) then
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink has cleared any prefix message.", 0, 1, 0)
			al.prefix = ""
			return
		end
		al.prefix = subcmd
		DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now prefix your messages with: ".. al.prefix, 0, 1, 0)
	elseif (cmd == "message" or cmd == "append") then
		if (subcmd == "" or subcmd == nil) then
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink has cleared any custom message.", 0, 1, 0)
			al.append = ""
			return
		end
		al.append = subcmd
		DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now append your custom message: ".. al.append, 0, 1, 0)
	elseif cmd == "ignore" then
		if (subcmd == "" or subcmd == nil) then
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink Ignore list:", 0, 1, 0)
			for name,status in pairs(al.ignore) do
				DEFAULT_CHAT_FRAME:AddMessage("    "..name, 0, 1, 0)
			end
			return
		end
		local name = strtrim(subcmd)
		name = string.lower(subcmd)
		name = string.gsub(name, "^%l", string.upper)
		if al.ignore[name] == nil then
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now ignore '"..name.."'.", 0, 1, 0)
			al.ignore[name] = true
		else
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will stopping ignoring '"..name.."'.", 0, 1, 0)
			al.ignore[name] = nil
		end
	elseif cmd == "debug" then
		if al.debug then
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will no longer show debugging messages.", 0, 1, 0)
			al.debug = false
		else
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink will now show debugging messages.", 0, 1, 0)
			al.debug = true
			AutoLink_Debug()
		end
	else
		AutoLink_SlashCmdList()
	end
end

function AutoLink_SlashCmdList()
	if al.enabled then
		DEFAULT_CHAT_FRAME:AddMessage("AutoLink is |cFF0000FFenabled|r.  Supported commands:", 0, 1, 0)
	else
		DEFAULT_CHAT_FRAME:AddMessage("AutoLink is |cFFFF0000disabled|r.  Supported commands:", 0, 1, 0)
		DEFAULT_CHAT_FRAME:AddMessage("  |cFFFFFFFFenable|r - enable AutoLink", 0, 1, 0)
		return
	end
	DEFAULT_CHAT_FRAME:AddMessage("  |cFFFFFFFFdisable|r - disable AutoLink", 0, 1, 0)
	DEFAULT_CHAT_FRAME:AddMessage("  |cFFFFFFFF<chat_channel>|r - toggle monitoring on that channel.", 0, 1, 0)
	DEFAULT_CHAT_FRAME:AddMessage("  |cFFFFFFFFall_chats|r - monitor all chat channels.", 0, 1, 0)
	DEFAULT_CHAT_FRAME:AddMessage("  |cFFFFFFFFall_skills|r - respond to requests to list all skills.", 0, 1, 0)
	DEFAULT_CHAT_FRAME:AddMessage("  |cFFFFFFFF<profession>|r - toggle linking that profession.", 0, 1, 0)
	DEFAULT_CHAT_FRAME:AddMessage("  |cFFFFFFFFlist|r - list status of channel monitoring, linking, and other options.", 0, 1, 0)
	DEFAULT_CHAT_FRAME:AddMessage("  |cFFFFFFFFself|r - toggle responding to your own messages.", 0, 1, 0)
	DEFAULT_CHAT_FRAME:AddMessage("  |cFFFFFFFFprefix <message>|r - Prefix message to all links (Default: (AutoLink).", 0, 1, 0)
	DEFAULT_CHAT_FRAME:AddMessage("  |cFFFFFFFFmessage <message>|r - Append a custom message to all links.", 0, 1, 0)
	DEFAULT_CHAT_FRAME:AddMessage("  |cFFFFFFFFalways|r - toggle responding during AFK/DND.", 0, 1, 0)
	DEFAULT_CHAT_FRAME:AddMessage("  |cFFFFFFFFignore <name>|r - add/remove a name from the AutoLink ignore list.", 0, 1, 0)
	DEFAULT_CHAT_FRAME:AddMessage("  |cFFFFFFFFnospam|r - toggle  anti-spam (1 min ban to each person) or allowing people to trigger you endlessles (careful)", 0, 1, 0)
	DEFAULT_CHAT_FRAME:AddMessage("  |cFFFFFFFFpublic|r - toggle responding via whispers or original channel the request came from.", 0, 1, 0)
	DEFAULT_CHAT_FRAME:AddMessage("  |cFFFFFFFFreset|r - reset all settings to default.", 0, 1, 0)
end

SlashCmdList["AUTOLINK"] = AutoLink_SlashCmdHandler
SLASH_AUTOLINK1 = "/AutoLink"
SLASH_AUTOLINK2 = "/autolink"
SLASH_AUTOLINK3 = "/AL"
SLASH_AUTOLINK4 = "/al"

function AutoLink_ClearRecent()
	local hour,min = GetGameTime()	-- Check if author has asked this minute
	for name,min_sent in pairs(linked) do
		if not min_sent == min then
			name = nil
		end
	end
end

function AutoLink(message, author, channel, report_on_error)
--	if (al.prefix~=nil and al.prefix~="") then
--		if string.find(message, al.prefix) then
--			if al.debug then DEFAULT_CHAT_FRAME:AddMessage("AutoLink DEBUG: Ignoring message because it was generated by AutoLink.", 0, 1, 0) end
--			return  --ignoring any messages sent by this mod
--		end
--	end
	if (author == UnitName("player") and not al.self) then
		if al.debug then DEFAULT_CHAT_FRAME:AddMessage("AutoLink DEBUG: Ignoring messages from you by request.", 0, 1, 0) end
		return  --ignoring any messages sent by the player
	end
	if UnitIsAFK("player") then
		if not al.always then
			if al.debug then DEFAULT_CHAT_FRAME:AddMessage("AutoLink DEBUG: You are AFK, clear your status to enable AutoLink.", 0, 1, 0) end
			return
		end
	end
	if UnitIsDND("player") then
		if not al.always then
			if al.debug then DEFAULT_CHAT_FRAME:AddMessage("AutoLink DEBUG: You are in DND, clear your status to enable AutoLink.", 0, 1, 0) end
			return
		end
	end
	if al.debug then DEFAULT_CHAT_FRAME:AddMessage("AutoLink DEBUG: Checking for ignore match on: ["..author.."].", 0, 1, 0) end
	if al.ignore[author] then
		if al.debug then DEFAULT_CHAT_FRAME:AddMessage("AutoLink DEBUG: Ignoring "..author.." as per settings.", 0, 1, 0) end
		return
	end
	if al.nospam then
		AutoLink_DebugChat("DEBUG: Anti-spam measures enabled, checking last request by "..author)
		local hour,min = GetGameTime()	-- Check if author has asked this minute
		if linked[author] == min then
			if al.debug then DEFAULT_CHAT_FRAME:AddMessage("AutoLink DEBUG: ["..author.."] already responded to at minute ["..min.."].", 0, 1, 0) end
			--DEFAULT_CHAT_FRAME:AddMessage("AutoLink: Temporarily ignoring "..author.." this minute to prevent spamming.", 0, 1, 0)
			return
		end
	end
	msg = strlower(message)
	if (string.find(msg, "lfw") or string.find(msg, "lf work") or string.find(msg, "wts") or string.find(msg, "looking for work") or string.find(msg, "lfm")) then
		return  --we don't want to respond to people advertising their own skills or LFM
	end
	if al.debug then DEFAULT_CHAT_FRAME:AddMessage("AutoLink DEBUG: Checking for !command matches.", 0, 1, 0) end
	local req = {}
	for request in string.gmatch(msg, "%!%a+") do	--Check for !<something> matches, store in req table (request)
		if al.debug then 
			--DEFAULT_CHAT_FRAME:AddMessage("AutoLink DEBUG: Checking for match on: ["..request.."]", 0, 1, 0)
		end
		if (string.find(request, "^!alch")) then req.alchemy = true end
		if (string.find(request, "^!blackmith") or string.find(request, "^!bs")) then req.bs = true end
		if (string.find(request, "^!cook")) then req.cooking = true end
		if (string.find(request, "^!enchant") or string.find(request, "^!chant")) then req.enchanting = true end
		if (string.find(request, "^!eng")) then req.eng = true end
		if (string.find(request, "^!first") or string.find(request, "^!aid")) then req.firstaid = true end
		if (string.find(request, "^!inscri") or string.find(request, "^!scri")) then req.inscription = true end
		if (string.find(request, "^!jewel") or string.find(request, "^!jc")) then req.jc = true end
		if (string.find(request, "^!leather") or string.find(request, "^!lw")) then req.lw = true end
		if (string.find(request, "^!tailor")) then req.tailoring = true end
		if (string.find(request, "^!link") or string.find(request, "^!autolink") or string.find(request, "^!show") or string.find(request, "!list")) then req.all = true end
	end
	local match_bang = false
	for tolink,status in pairs(req) do	--check if any matches were found
		if al.debug then 
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink DEBUG: !command match found for: ".. tolink, 0, 1, 0)
		end
		match_bang = true
	end
	if not match_bang then	-- No !command found, look for combo of trigger words and skill requests
		local match_trigger = false
		local match_request = false
		if al.debug then 
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink DEBUG: No !command requests, looking for word matches.", 0, 1, 0)
		end
		triggers = {
				"link",
				"show",
				"list",
				"lf"
			}
		for request in string.gmatch(msg, "%a+") do	--Check for request matches, store in req table
			for i,word in pairs(triggers) do
				if al.debug then 
					--DEFAULT_CHAT_FRAME:AddMessage("AutoLink DEBUG: Checking for match on: ["..request.."] against ["..word.."]", 0, 1, 0)
				end
				if request == word then 
					match_trigger = true 
					if al.debug then 
						DEFAULT_CHAT_FRAME:AddMessage("AutoLink DEBUG: Match found: ["..request.."]", 0, 1, 0)
					end
					break
				end
			end
			if (string.find(request, "^alch")) then 
				req.alchemy = true
				match_request = true
				break
			elseif (string.find(request, "^blacksmith") or string.find(request, "^bs") or string.find(request, "^bsmith") or string.find(request, "^bser")) then 
				req.bs = true
				match_request = true
				break
			elseif (string.find(request, "^cook")) then 
				req.cooking = true
				match_request = true
				break
			elseif (string.find(request, "^chant") or string.find(request, "^enchant") or string.find(request, "^chant") or string.find(request, "^ench")) then 
				req.enchanting = true
				match_request = true
				break
			elseif (string.find(request, "^eng")) then 
				req.eng = true
				match_request = true
				break
			elseif (string.find(request, "^first aid") or string.find(request, "^firstaid")) then 
				req.firstaid = true
				match_request = true
				break
			elseif (string.find(request, "^inscri") or string.find(request, "^scri")) then 
				req.inscription = true
				match_request = true
				break
			elseif (string.find(request, "^jewel") or string.find(request, "^jc")) then 
				req.jc = true
				match_request = true
				break
			elseif (string.find(request, "^leather") or string.find(request, "^lw")) then 
				req.lw = true
				match_request = true
				break
			elseif (string.find(request, "^tailor")) then 
				req.tailoring = true
				match_request = true
				break
			elseif (string.find(request, "^professions") or string.find(request, "^skills") or string.find(request, "^all") or string.find(request, "^skillz")) then 
				if al.all then
					req.all = true
					match_request = true
					break
				else
					match_request = false
				end
			else 
				match_request = false
			end
		end
		if not (match_trigger and match_request) then return end
	end
	if (match_bang or (match_trigger and match_request)) then 
		local match = true 
	else 
		local match = false 
	end
	for tolink,status in pairs(req) do	--check if any matches were found
		if al.debug then 
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink DEBUG: trigger word match found for: ".. tolink, 0, 1, 0)
		end
		match = true
	end
	if match then
		local linked_someone = false
		if al.debug then 
			DEFAULT_CHAT_FRAME:AddMessage("AutoLink DEBUG: Match found, checking if link needs to be sent.", 0, 1, 0)
			for tolink,status in pairs(req) do
				DEFAULT_CHAT_FRAME:AddMessage("AutoLink DEBUG: Checking for link of: ".. tolink, 0, 1, 0)
			end
		end
		if req.all then
			if not al.all then return end		--just in case, if they have toggled this off, exit
			--Due to some limitation in WoW, only two professions can be linked at once.  So split it into primary/secondary
			local skills = AutoLink_GetAllCrafts()
			if (skills==nil or skills=="") then 		--no professions, or all toggled off
				AutoLink_RespondChat(al.prefix .. " I am too lazy to train any professions.", channel, author)
			else
				linked_someone = true
				if (al.append~=nil and al.append~="") then
					AutoLink_RespondChat(al.prefix .. al.append, channel, author)
				end
				for i,skill in pairs(skills) do
					AutoLink_RespondChat(al.prefix .. skill, channel, author)
				end
			end
		elseif req.alchemy then
			if (IsUsableSpell("Alchemy") and al.alchemy) then
				skill = AutoLink_GetSkill("Alchemy")
				if not skill then
					AutoLink_DebugChat("DEBUG: Alchemy requested, but not found.")
				else
					linked_someone = true
					AutoLink_RespondChat(al.prefix .. skill .. al.append, channel, author)
				end
			else
				if report_on_error then
					AutoLink_NoSkill("Alchemy", author)
				end
			end
		elseif req.bs then
			if (IsUsableSpell("Blacksmithing") and al.bs) then
				skill = AutoLink_GetSkill("Blacksmithing")
				if not skill then
					AutoLink_DebugChat("DEBUG: Blacksmithing requested, but not found.")
				else
					linked_someone = true
					AutoLink_RespondChat(al.prefix .. skill .. al.append, channel, author)
				end
			else
				if report_on_error then
					AutoLink_NoSkill("Blacksmithing", author)
				end
			end
		elseif req.enchanting then
			if (IsUsableSpell("Enchanting") and al.enchanting) then
				skill = AutoLink_GetSkill("Enchanting")
				if not skill then
					AutoLink_DebugChat("DEBUG: Enchanting requested, but not found.")
				else
					linked_someone = true
					AutoLink_RespondChat(al.prefix .. skill .. al.append, channel, author)
				end
			else
				if report_on_error then
					AutoLink_NoSkill("Enchanting", author)
				end
			end
		elseif req.eng then
			if (IsUsableSpell("Engineering") and al.eng) then
				skill = AutoLink_GetSkill("Engineering")
				if not skill then
					AutoLink_DebugChat("DEBUG: Engineering requested, but not found.")
				else
					linked_someone = true
					AutoLink_RespondChat(al.prefix .. skill .. al.append, channel, author)
				end
			else
				if report_on_error then
					AutoLink_NoSkill("Engineering", author)
				end
			end
		elseif req.inscription then
			if (IsUsableSpell("Inscription") and al.inscription) then
				skill = AutoLink_GetSkill("Inscription")
				if not skill then
					AutoLink_DebugChat("DEBUG: Inscription requested, but not found.")
				else
					linked_someone = true
					AutoLink_RespondChat(al.prefix .. skill .. al.append, channel, author)
				end
			else
				if report_on_error then
					AutoLink_NoSkill("Inscription", author)
				end
			end
		elseif req.jc then
			if (IsUsableSpell("Jewelcrafting") and al.jc) then
				skill = AutoLink_GetSkill("Jewelcrafting")
				if not skill then
					AutoLink_DebugChat("DEBUG: Jewelcrafting requested, but not found.")
				else
					linked_someone = true
					AutoLink_RespondChat(al.prefix .. skill .. al.append, channel, author)
				end
			else
				if report_on_error then
					AutoLink_NoSkill("Jewelcrafting", author)
				end
			end
		elseif req.lw then
			if (IsUsableSpell("Leatherworking") and al.lw) then
				skill = AutoLink_GetSkill("Leatherworking")
				if not skill then
					AutoLink_DebugChat("DEBUG: Leatherworking requested, but not found.")
				else
					linked_someone = true
					AutoLink_RespondChat(al.prefix .. skill .. al.append, channel, author)
				end
			else
				if report_on_error then
					AutoLink_NoSkill("Leatherworking", author)
				end
			end
		elseif req.tailoring then
			if (IsUsableSpell("Tailoring") and al.tailoring) then
				skill = AutoLink_GetSkill("Tailoring")
				if not skill then
					AutoLink_DebugChat("DEBUG: Tailoring requested, but not found.")
				else
					linked_someone = true
					AutoLink_RespondChat(al.prefix .. skill .. al.append, channel, author)
				end
			else
				if report_on_error then
					AutoLink_NoSkill("Tailoring", author)
				end
			end
		elseif req.cooking then
			if (IsUsableSpell("Cooking") and al.cooking) then
				skill = AutoLink_GetSkill("Cooking")
				if not skill then
					AutoLink_DebugChat("DEBUG: Cooking requested, but not found.")
				else
					linked_someone = true
					AutoLink_RespondChat(al.prefix .. skill .. al.append, channel, author)
				end
			else
				if report_on_error then
					AutoLink_NoSkill("Cooking", author)
				end
			end
		elseif req.firstaid then
			if (IsUsableSpell("First Aid") and al.firstaid) then
				skill = AutoLink_GetSkill("First Aid")
				if not skill then
					AutoLink_DebugChat("DEBUG: First Aid requested, but not found.")
				else
					linked_someone = true
					AutoLink_RespondChat(al.prefix .. skill .. al.append, channel, author)
				end
			else
				if report_on_error then
					AutoLink_NoSkill("First Aid", author)
				end
			end
		end
		if al.nospam then
			AutoLink_DebugChat("DEBUG: Anti-spam measures enabled, recording time of request for "..author)
			if linked_someone then
				AutoLink_ClearRecent()
				local hour,min = GetGameTime()	-- Record time that author requested link, for minor spam prevention
				if al.debug then 
					DEFAULT_CHAT_FRAME:AddMessage("AutoLink DEBUG: Recording current minute of ["..min.."] for sender: ["..author.."].", 0, 1, 0)
				end
				linked[author] = min
			end
		end
	end
end

function AutoLink_OnEvent(this, event, arg1, arg2, arg3, arg4, ...)
	if(event == "ADDON_LOADED" and arg1 == "AutoLink") then
		AutoLink_Load()
	elseif not al.enabled then
		return
	elseif event == "CHAT_MSG_WHISPER" then
		AutoLink(arg1, arg2, "WHISPER", true)
	elseif event == "CHAT_MSG_GUILD" then
		if not al.guild then return end
		AutoLink(arg1, arg2, "GUILD", false)
	elseif (event == "CHAT_MSG_PARTY" or event == "CHAT_MSG_PARTY_LEADER") then
		if not al.party then return end
		AutoLink(arg1, arg2, "PARTY", false)
	elseif (event == "CHAT_MSG_RAID" or event == "CHAT_MSG_RAID_LEADER") then
		if not al.raid then return end
		AutoLink(arg1, arg2, "RAID", false)
	elseif event == "CHAT_MSG_SAY" then
		if not al.say then return end
		AutoLink(arg1, arg2, "SAY", false)
	elseif event == "CHAT_MSG_YELL" then
		if not al.yell then return end
		AutoLink(arg1, arg2, "YELL", false)
	elseif event == "CHAT_MSG_CHANNEL" then
		if string.find(arg4, "Trade") then
			if not al.trade then
				return 
			else
				AutoLink(arg1, arg2, "Trade - City", false)
			end
		elseif string.find(arg4, "General") then
			if not al.general then
				return
			else
				AutoLink(arg1, arg2, "General", false)
			end
		end
	end
end

ALplayername, ALrealm = UnitName("player")

local frame = CreateFrame("Frame", "AutoLinkFrame")
frame:SetScript("OnEvent", AutoLink_OnEvent)
frame:RegisterEvent("ADDON_LOADED")
frame:RegisterEvent("CHAT_MSG_WHISPER")
frame:RegisterEvent("CHAT_MSG_GUILD")
frame:RegisterEvent("CHAT_MSG_PARTY")
frame:RegisterEvent("CHAT_MSG_PARTY_LEADER")
frame:RegisterEvent("CHAT_MSG_RAID")
frame:RegisterEvent("CHAT_MSG_RAID_LEADER")
frame:RegisterEvent("CHAT_MSG_SAY")
frame:RegisterEvent("CHAT_MSG_YELL")
frame:RegisterEvent("CHAT_MSG_CHANNEL")
--frame:RegisterEvent("CHAT_MSG_WHISPER_INFORM")
